=== modified file 'www/media/browser/browser.js'
--- www/media/browser/browser.js	2008-02-18 02:17:47 +0000
+++ www/media/browser/browser.js	2008-02-25 00:02:33 +0000
@@ -123,16 +123,20 @@
 function do_action(action, path, args, content_type, ignore_response)
 {
     args.action = action;
+    /* Callback action, when the server returns */
+    var callback = function(response)
+        {
+            /* Check for action errors reported by the server, and report them
+             * to the user */
+            var error = response.getResponseHeader("X-IVLE-Action-Error");
+            if (error != null)
+                alert("Error: " + error.toString() + ".");
+            /* Now read the response and set up the page accordingly */
+            if (ignore_response != true)
+                handle_response(path, response);
+        }
     /* Call the server and perform the action. This mutates the server. */
-    response = ajax_call(service_app, path, args, "POST", content_type);
-    /* Check for action errors reported by the server, and report them to the
-     * user */
-    error = response.getResponseHeader("X-IVLE-Action-Error");
-    if (error != null)
-        alert("Error: " + error.toString() + ".");
-    /* Now read the response and set up the page accordingly */
-    if (ignore_response != true)
-        handle_response(path, response);
+    ajax_call(callback, service_app, path, args, "POST", content_type);
 }
 
 /** Calls the server using Ajax, requesting a directory listing. This should
@@ -147,10 +151,13 @@
  */
 function navigate(path, editmode)
 {
+    callback = function(response)
+        {
+            /* Read the response and set up the page accordingly */
+            handle_response(path, response, editmode);
+        }
     /* Call the server and request the listing. This mutates the server. */
-    response = ajax_call(service_app, path, null, "GET");
-    /* Now read the response and set up the page accordingly */
-    handle_response(path, response, editmode);
+    ajax_call(callback, service_app, path, null, "GET");
 }
 
 /** Determines the "handler type" from a MIME type.

=== modified file 'www/media/common/tos.js'
--- www/media/common/tos.js	2008-02-18 23:59:17 +0000
+++ www/media/common/tos.js	2008-02-25 00:02:33 +0000
@@ -73,9 +73,8 @@
         "IVLE is now setting up your environment. Please wait..."));
     tos_acceptbuttons.appendChild(make_dots_anim());
     /* Make the Ajax request */
-    var xhr = ajax_call("userservice", "activate_me", USER_DECLARATION,
-        "POST")
-    handle_accept_response(xhr)
+    ajax_call(handle_accept_response, "userservice", "activate_me",
+        USER_DECLARATION, "POST")
 }
 
 function handle_accept_response(xhr)

=== modified file 'www/media/common/util.js'
--- www/media/common/util.js	2008-02-24 23:16:42 +0000
+++ www/media/common/util.js	2008-02-25 00:02:33 +0000
@@ -551,10 +551,13 @@
     }
 }
 
-/** Makes an XMLHttpRequest call to the server. Waits (synchronously) for a
- * response, and returns an XMLHttpRequest object containing the completed
- * response.
+/** Makes an asynchronous XMLHttpRequest call to the server.
+ * Sends the XMLHttpRequest object containing the completed response to a
+ * specified callback function.
  *
+ * \param callback A callback function. Will be called when the response is
+ *      complete. Passed 1 parameter, an XMLHttpRequest object containing the
+ *      completed response.
  * \param app IVLE app to call (such as "fileservice").
  * \param path URL path to make the request to, within the application.
  * \param args Argument object, as described in parse_url and friends.
@@ -562,9 +565,8 @@
  * \param content_type String, optional. Only applies if method is "POST".
  *      May be "application/x-www-form-urlencoded" or "multipart/form-data".
  *      Defaults to "application/x-www-form-urlencoded".
- * \return An XMLHttpRequest object containing the completed response.
  */
-function ajax_call(app, path, args, method, content_type)
+function ajax_call(callback, app, path, args, method, content_type)
 {
     if (content_type != "multipart/form-data")
         content_type = "application/x-www-form-urlencoded";
@@ -575,20 +577,26 @@
      * used within this function) */
     var boundary = "48234n334nu7n4n2ynonjn234t683jyh80j";
     var xhr = new_xmlhttprequest();
+    xhr.onreadystatechange = function()
+        {
+            if (xhr.readyState == 4)
+            {
+                callback(xhr);
+            }
+        }
     if (method == "GET")
     {
         /* GET sends the args in the URL */
         url = build_url({"path": path, "args": args});
-        /* open's 3rd argument = false -> SYNCHRONOUS (wait for response)
-         * (No need for a callback function) */
-        xhr.open(method, url, false);
+        /* open's 3rd argument = true -> asynchronous */
+        xhr.open(method, url, true);
         xhr.send(null);
     }
     else
     {
         /* POST sends the args in application/x-www-form-urlencoded */
         url = encodeURI(path);
-        xhr.open(method, url, false);
+        xhr.open(method, url, true);
         var message;
         if (content_type == "multipart/form-data")
         {
@@ -604,6 +612,5 @@
         xhr.setRequestHeader("Content-Length", message.length);
         xhr.send(message);
     }
-    return xhr;
 }
 

=== modified file 'www/media/console/console.js'
--- www/media/console/console.js	2008-02-06 03:21:36 +0000
+++ www/media/console/console.js	2008-02-25 00:02:33 +0000
@@ -36,18 +36,32 @@
 
 /* Starts the console server, if it isn't already.
  * This can be called any number of times - it only starts the one server.
+ * Note that this is asynchronous. It will return after signalling to start
+ * the server, but there is no guarantee that it has been started yet.
  * This is a separate step from console_init, as the server is only to be
  * started once the first command is entered.
  * Does not return a value. Writes to global variables
  * server_host, and server_port.
+ *
+ * \param callback Function which will be called after the server has been
+ * started. No parameters are passed. May be null.
  */
-function start_server()
+function start_server(callback)
 {
-    if (server_started) return;
-    var xhr = ajax_call("consoleservice", "start", {}, "POST");
-    var json_text = xhr.responseText;
-    server_key = JSON.parse(json_text);
-    server_started = true;
+    if (server_started)
+    {
+        callback();
+        return;
+    }
+    var callback1 = function(xhr)
+        {
+            var json_text = xhr.responseText;
+            server_key = JSON.parse(json_text);
+            server_started = true;
+            if (callback != null)
+                callback();
+        }
+    ajax_call(callback1, "consoleservice", "start", {}, "POST");
 }
 
 /** Initialises the console. All apps which import console are required to
@@ -203,12 +217,17 @@
  */
 function console_enter_line(inputline, which)
 {
-    /* Start the server if it hasn't already been started */
-    start_server();
     var args = {"key": server_key, "text":inputline};
-    var xmlhttp = ajax_call("consoleservice", which, args, "POST");
+    var callback = function(xhr)
+        {
+            console_response(inputline, xhr.responseText);
+        }
+    ajax_call(callback, "consoleservice", which, args, "POST");
+}
 
-    var res = JSON.parse(xmlhttp.responseText);
+function console_response(inputline, responseText)
+{
+    var res = JSON.parse(responseText);
     var output = document.getElementById("console_output");
     {
         var pre = document.createElement("pre");
@@ -304,10 +323,15 @@
          */
         break;
     case 13:                /* Enter key */
-        /* Send the line of text to the server */
-        console_enter_line(inp.value, "chat");
-        hist.submit(inp.value);
-        inp.value = hist.curr();
+        var callback = function()
+        {
+            /* Send the line of text to the server */
+            console_enter_line(inp.value, "chat");
+            hist.submit(inp.value);
+            inp.value = hist.curr();
+        }
+        /* Start the server if it hasn't already been started */
+        start_server(callback);
         break;
     case 38:                /* Up arrow */
         hist.up(inp.value);

=== modified file 'www/media/tutorial/tutorial.js'
--- www/media/tutorial/tutorial.js	2008-02-22 00:01:34 +0000
+++ www/media/tutorial/tutorial.js	2008-02-25 00:02:33 +0000
@@ -67,10 +67,14 @@
 
     /* Send the form as multipart/form-data, since we are sending a whole lump
      * of Python code, it should be treated like a file upload. */
-    var xhr = ajax_call("tutorialservice", "", args, "POST",
+    /* AJAX callback function */
+    var callback = function(xhr)
+        {
+            var testresponse = JSON.parse(xhr.responseText);
+            handle_testresponse(exercisediv, testresponse);
+        }
+    ajax_call(callback, "tutorialservice", "", args, "POST",
         "multipart/form-data");
-    var testresponse = JSON.parse(xhr.responseText);
-    handle_testresponse(exercisediv, testresponse);
 }
 
 /** Given a exercise div, return the testoutput div which is its child.

